(define-module (alterator configd)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-13)

  :use-module (alterator str)
  :use-module (alterator algo)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :use-module (alterator gettext)

  :use-module (alterator http html)
  :use-module (alterator http template)

  :use-module (alterator configd frontend)
  :use-module (alterator configd translate)
  :use-module (alterator configd workflow)
  :use-module (alterator configd html)
  :use-module (alterator configd woo)

  :export (configd))

(define (other-option? x)
  (not (eq? (car x) 'value)))

(define (translate-value options translator)
  (let ((value (cond-assq 'value options)))
    (if value
        (acons 'value (translator value)
               (filter other-option? options))
        options)))

(define (process-module action url url-args)
  (or (string=? url "/")
      (process-workflow action
                        url
                        url-args
                        (process-template url url-args))))

(define (process-framework action url scm2 url-args)
  (process-workflow action
                    url
                    url-args
		    (acons 'scm2 scm2 (process-template "/" '()))))

(define (form-action method url-args)
  (cond
    ((string-ci=? "post" method)
      (cond-assoc "action" url-args "write"))
    ((string-ci=? "get" method)
      "read")
    (else #f)))

(define (configd-main method url url-args . rest)
    (catch #t
	   (thunk
	     (cond
	       ((form-action method url-args) =>
	        (lambda (action)
	           (process-translate
	              (process-framework action
					 url
					 (process-module action url url-args)
	                                 url-args)
	              url
	              url-args)))
	       ((string-ci=? method "woo")
		(apply woo-try url-args url rest))
	       (else (write '() ))))
	   (html:exception url-args)))

(define (configd)
  (lambda (cmds next)
    (catch #t
      (thunk
       (with-fluids
        ((woo-gate next))
        (apply configd-main (car cmds))))
      (html:exception '() ))))
