(define-module (alterator configd form)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-13)
  :use-module (alterator plist)
  :use-module (alterator algo)
  :use-module (alterator str)
  :use-module (alterator woo)
  :use-module (alterator constraints)
  :use-module (alterator http template)
  :use-module (alterator http html)
  :use-module (alterator http utils)
  :use-module (alterator configd html)
  :use-module (alterator configd constraints)
  :use-module (alterator configd woo)
 :export (;;high level
          workflow-form
          fill-form
          new-form
	  ;;low level
	  process-form))

;;; main form processing

(define (main-args url url-args)
  (let ((args (woo-args url url-args)))
    (or (find (lambda(arg) (string=? url (car arg))) args)
	(car (woo-args url (list  (assoc "language" url-args)))))))

(define (not-name? name)
  (lambda(x) (not (string=? (car x) name))))

(define (redirect-args name-attribute url-args)
  (let ((name-value (cond-assoc "name" url-args)))
    (if (and name-attribute name-value)
      (acons name-attribute name-value
             (filter (not-name? name-attribute) url-args))
      url-args)))

(define (redirect-form url url-args template-args)
  (let* ((async (cond-assoc "async" url-args))
	 (redirect-url (or (cond-assq 'redirect template-args)
			   (cond-assq 'redirect-url template-args)
			   url))
	 (name-attribute (cond-assq 'name-attribute template-args))
	 (url-args (redirect-args name-attribute url-args)))
    (if (or (string-null? redirect-url) async name-attribute)
      (workflow-form url "read" url-args template-args)
      (html:redirect redirect-url))))

(define (workflow-form url action url-args template-args)
  (let ((scm (cond-assq 'scm template-args))
	(backend-url (cond-assq 'backend-url template-args url))
	(url-redirect (cond-assq 'url-redirect template-args #f)))
    (cond
      (url-redirect
	(html:redirect url-redirect))
      ((string=? action "read")
       (let ((args (main-args backend-url url-args)))
	 (template scm
		   (apply fill-constraints "write" args)
		   (apply fill-form args))))
      (else
	(for-each
	  (lambda(cmd)
	    (apply (if (string=? action "delete") woo-try woo-try/constraints)
		   action cmd))
	  (woo-args backend-url url-args))
	(redirect-form url url-args template-args)))))

;;; form fill

(define (fill-form obj-url . obj-args)
  (process-form (delay (apply woo-read-first obj-url obj-args))
		obj-url obj-args))

(define (get-defaults obj-url obj-args)
  (woo-catch
    (thunk
      (process-defaults
	(read-constraints "new" obj-url obj-args)
	'() ))
    (lambda args '() )))

(define (new-form obj-url . obj-args)
  (let ((fake-url (string-append obj-url "/.object")))
    (process-form (delay (cons fake-url (get-defaults obj-url obj-args)))
		  fake-url obj-args)))

(define (process-form obj obj-url obj-args . obj-sub-url)
  (let ((obj-sub-url (if (null? obj-sub-url) "" (car obj-sub-url))))
    (list
     ;;anchors
     (process-a obj obj-url obj-args)

     ;;spans
     (process-span obj)

     ;;images
     (process-img obj)

     ;;divs
     (process-progress obj)
     (process-text obj)
     (process-timeedit obj)
     (process-dateedit obj)

     ;; meta
     (process-refresh obj)

     ;; selects
     (process-select obj obj-url obj-args obj-sub-url)

     ;; inputs
     (process-hidden obj obj-sub-url)
     (process-checkbox obj obj-sub-url)
     (process-edit obj obj-sub-url)

     ;; textareas
     (process-textarea obj obj-sub-url)

     ;; tables
     (process-checklistbox obj obj-args)
     (process-listbox2 obj-url obj-args)
     (and (string-null? obj-sub-url) (process-tbody obj-url obj-args)) )))

;;; helpers

(define (fill-content content obj obj-url obj-args . obj-sub-url)
  (template
    content
    (apply process-form obj obj-url obj-args obj-sub-url)))

(define (get-value options obj . default)
  (let* ((name (cond-assq 'name options ""))
         (prev-value (cond-assq 'value options ""))
         (default (if (pair? default) (car default) prev-value)))
    (woo-get-option (force obj) (string->symbol name) default)))

(define (other-option? x)
  (not (memq (car x) '(name value href checked))))

(define (tag-options options obj-sub-url)
  (let ((name (cond-assq 'name options "")))
    (acons 'name
           (if (string-null? obj-sub-url)
               name
               (string-append obj-sub-url ":" name))
          (filter other-option? options))))

;;; selects

(define (select-url obj-url options)
  (let ((optionlist-attribute (cond-assq 'optionlist options))
	(enumref-attribute (cond-assq 'enumref options)))
    (or enumref-attribute
	(and (string? optionlist-attribute)
	     (string-append obj-url "/" optionlist-attribute)))))

(define (process-select obj obj-url obj-args obj-sub-url)
  (replace-tag: "select"
		(lambda (options content)
		  (let* ((variant-url (select-url obj-url options)))
		    (if (string? variant-url)
		      `(select ,@(tag-options options obj-sub-url)
			       ,(html:select-options (get-value options obj)
						     (apply woo-list variant-url obj-args)))
		      `(select ,@options ,@content))))))

;;; anchors

(define (process-a obj obj-url obj-args)
  (list
    (replace-tag: "a"
		  (@ 'class "alterator-ref2")
		  (lambda (options content)
		    (let ((ref-obj (encode-url-component (woo-get-option (force obj) 'name)))
			  (ref-url (cond-assq 'href options)))
		      `(a ,(@ 'href (string-append ref-url ref-obj))
			  ,@(filter other-option? options)
			  ,(fill-content content obj obj-url obj-args)))))
    (replace-tag: "a"
		  (@ 'class "alterator-ref")
		  (lambda(options content)
		    (let ((ref-obj (car (force obj)))
			  (ref-url (cond-assq 'href options)))
		      `(a ,(@ 'href (string-append ref-obj ref-url))
			  ,@(filter other-option? options)
			  ,(fill-content content obj obj-url obj-args)))))))

;;; spans

(define (process-span obj)
  (replace-tag: "span"
                (@ 'class "alterator-label")
                (lambda(options content)
		  `(span ,@options ,(->string (get-value options obj))))))


;;; images

(define (process-img obj)
  (replace-tag: "img"
                (@ 'class "alterator-img")
                (lambda(options content)
		  `(img ,(@ 'src
			    (string-append "/design/images/" (->string (get-value options obj)) ".png" ))))))

;;; divs

(define (process-progress obj)
  (replace-tag: "div"
                (@ 'class "alterator-progress")
                (lambda(options content)
                  `(div ,@options
                        ,(html: "div"
                                (@ 'class "alterator-progress-slider")
                                (@ 'style (format #f "width:~A%" (get-value options obj))))))))

;;; add refresh to meta tag

(define (process-refresh obj)
  (replace-tag: "meta"
                (@ 'class "alterator-refresh")
                (lambda(options content)
                  (let ((rate (woo-get-option (force obj) 'refresh #f)))
                    (if rate `(meta ,(@ 'http-equiv "refresh") ,(@ 'content rate))
                             `(meta) )))))

(define (html-body url)
 (and (string? url)
      (call-with-current-continuation
        (lambda(return)
          (template
            url
            (tag: "body" (lambda (options content) (return content))))
          #f))))

(define (process-text obj)
  (replace-tag: "div"
                (@ 'class "alterator-text")
                (lambda(options content)
		  `(div ,@options ,@(or (html-body (get-value options obj))
		                        content)))))

(define (process-timeedit obj)
  (replace-tag: "div"
		(@ 'class "alterator-timeedit")
		(lambda(options content)
		  (let ((name (cond-assq 'name options "")))
		    (html: "div"
			   (html: "canvas" (@ 'id (string-append name "-canvas")))
			   (html: "br")
			   (html: "br")
			   (html: "input" (@ 'type "text")
				  (@ 'class "text")
				  (@ 'name name) (@ 'value (get-value options obj)))
			   (html: "script" (format #f "teCreateOnLoad(\"~A\");" name)))))))

(define (process-dateedit obj)
  (list
    (replace-tag: "div"
		  (@ 'class "alterator-dateedit")
		  (lambda(options content)
		    (let ((name (cond-assq 'name options "")))
		      (html: "div"
			     (@ 'class "alterator-dateedit")
			     (html: "div"
				    (@ 'id (string-append name "-calendar"))
				    (@ 'class "alterator-dateedit-calendar"))
			     (html: "br")
			     (html: "input"
				    (@ 'type "text")
				    (@ 'class "text")
				    (@ 'name name)
				    (@ 'value (get-value options obj)))
			     (html: "script" (format #f "deCreateCalendar(\"~A\");" name))))))
    (replace-tag: "span"
		  (@ 'class "alterator-dateedit")
		  (lambda (options content)
		    (let ((name (cond-assq 'name options "")))
		      (html: "span"
			     (@ 'class "alterator-dateedit")
			     (html: "input"
				    (@ 'type "text")
				    (@ 'class "text")
				    (@ 'name name)
				    (@ 'value (get-value options obj)))
			     (html: "img"
				    (@ 'id (string-append name "-button"))
				    (@ 'src "/design/images/null.gif")
				    (@ 'class "alterator-dateedit-button"))
			     (html: "div"
				    (@ 'id (string-append name "-calendar"))
				    (@ 'class "alterator-dateedit-calendar"))
			     (html: "script" (format #f "deCreatePicker(\"~A\");" name))))))))


;;; inputs

(define (form-error . msg)
  (throw 'form-error (apply format #f msg)))

(define (process-hidden obj obj-sub-url)
  (replace-tag: "input"
		(@ 'type "hidden")
		(lambda (options content)
		  (let ((name (cond-assq 'name options "unknown"))
			(value (get-value options obj)))
		    (cond
		      ((or (string? value) (number? value))
		       `(input ,@(tag-options options obj-sub-url)
			       (value . ,(->string value))))
		      (else
			(form error "hidden input (~S): expected string or number, got ~S" name value)))))))

(define (process-checkbox obj obj-sub-url)
  (replace-tag: "input"
		(@ 'type "checkbox")
		(lambda (options content)
		  (let* ((name (cond-assq 'name options "unknown"))
			 (prev-checked (string=? "yes" (cond-assq 'checked options  "no")))
			 (value (get-value options obj prev-checked)))
		    (cond
		      ((boolean? value)
		       `(input ,@(tag-options options obj-sub-url)
			       ,(if value (@ 'checked "yes") "")
			       (value . "#t")))
		      ((string? value)
		       `(input ,@(tag-options options obj-sub-url)
			       (value . ,value)))
		      (else
			(form-error "checkbox (~S): expected string or boolean, got ~S" name value)))))))

(define (process-edit obj obj-sub-url)
  (replace-tag: "input"
		(@ 'type "text")
		(lambda(options content)
		  (let ((name (cond-assq 'name options "unknown"))
			(value (get-value options obj)))
		    (cond
		      ((or (string? value) (number? value))
		       `(input ,@(tag-options options obj-sub-url)
			       (value . ,(->string value))))
		      (else
			(form-error "text input (~S): expected string or number, got ~S" name value)))))))


;;; textareas

(define (process-textarea obj obj-sub-url)
  (replace-tag: "textarea"
                (lambda(options content)
		  (let ((name (cond-assq 'name options "unknown"))
			(value (get-value options obj)))
		    (cond
		      ((string? value)
		       `(textarea ,@(tag-options options obj-sub-url)
				  ,value))
		      (else
			(form-error "text area (~S): expected string, got ~S" name value)))))))

;;; tables

(define (fill-row row obj parent-url obj-url obj-args)
  (fill-content row obj (string-append parent-url "/" obj-url) obj-args obj-url))

(define (process-tbody obj-url obj-args)
  (replace-tag: "tbody"
                (@ 'class "alterator-list")
                (lambda(options content)
                  (let* ((optionlist (cond-assq 'optionlist options ""))
                         (variants  (apply woo-list (string-append obj-url "/" optionlist) obj-args)))
                  `(tbody ,@options
                          ,@(map (lambda(obj)
                                   (fill-row content
                                             (delay obj)
					     obj-url
                                             (string-append optionlist
                                                            "/"
                                                            (cond-plistq 'name (cdr obj)))
					     obj-args))
                                 variants))))))

(define (process-listbox2-content scm obj-url obj-args variant-list)
  (template
    scm
    (replace-tag: "tbody"
		  (lambda(options content)
		    `(tbody ,@options
			    ,@(map (lambda(obj)
				     (fill-content content
						   (delay obj)
						   obj-url
						   obj-args))
				   variant-list))))))

(define (process-listbox2 obj-url obj-args)
  (replace-tag: "table"
		(@ 'class "alterator-listbox")
		(lambda(options content)
		  (let* ((variant-url (cond-assq 'enumref options ""))
			 (variant-list (apply woo-list variant-url obj-args)))
		    `(table ,@options
			    ,@(process-listbox2-content content
							obj-url
							obj-args
							variant-list))))))

(define (process-checklistbox obj obj-args)
  (replace-tag: "table"
		(@ 'class "alterator-checklistbox")
		(lambda(options content)
		  (let* ((name (cond-assq 'name options))
		         (variant-url (cond-assq 'enumref options))
			 (variant-list  (apply woo-list/name+label variant-url obj-args))
			 (value-list (string-cut (get-value options obj) #\;)))
		    `(table ,@options
			    ,(map (lambda(x)
				    (html: "tr"
					   (html: "td"
						  (html: "input" (@ 'type "checkbox")
						                 (if (member (car x) value-list) (@ 'checked "yes") "")
						                 (@ 'name name)
						                 (@ 'value (car x)))
						  (cdr x))))
				  variant-list))))))
