(define-module (alterator configd frontend)
  :use-module (srfi srfi-13)
  :use-module (srfi srfi-2)
  :use-module (alterator str)
  :use-module (alterator algo)
  :use-module (alterator plist)
  :use-module (alterator http template)
  :use-module (alterator http path)
  :use-module (alterator http utils)

  :use-module (alterator woo)
  :use-module (alterator configd woo)

  :export (process-template))

(define *index-list* '("index.html" "index.htm" "index.xhtml"))

;;; new style template processing

(define (new-template-wf scm)
  (call-with-current-continuation
    (lambda(return)
      (template scm
		(tag: "html"
		      (lambda(options content)
			(return (cond-assq 'wf options "form"))))))))

(define (new-template-port url)
  (let loop ((dirs (map (lambda (x) (string-append x "/templates/"))
			(string-cut (getenv "ALTERATOR_DATADIR") #\:))))
    (and (pair? dirs)
	 (or (open-path url (car dirs) *index-list*)
	     (loop (cdr dirs))))))

(define (new-template-args port)
  (and-let* (((port? port))
	     (scm  (port->scm port))
	     (wf   (string-split (new-template-wf scm) #\?))
	     (wf-name (car wf))
	     (wf-args (decode-url (or (cond-cadr wf) ""))))
	    (cons* (cons 'scm scm)
	           (cons 'template wf-name)
		   (map (lambda(x) (cons (string->symbol (car x)) (cdr x)))
			wf-args))))

(define (new-template url)
  (let* ((port (new-template-port url))
	 (args (new-template-args port)))
    (and (port? port) (close-port port))
    args))

;;; old style template processing

(define (old-template-port url)
  (and-let* (((not-empty-string? url))
	     (file (with-first-readable
		     (map (lambda(x) (string-append x "/html/fbi/" url))
			  (string-cut (getenv "ALTERATOR_HTMLDIR") #\:))
		     values)))
	    (file->scm file)))

(define (old-template-backend url)
  (string-append "/template-" (string-trim url #\/)))

(define (old-template-args url-args)
  (let ((action (cond-assoc "action" url-args "unknown")))
    (cons* 'orig_action action (alist->plist (filter-args url-args)))))

(define (old-template url url-args)
  (let* ((result (cond-car (apply woo-try "template" (old-template-backend url) (old-template-args url-args))))
	 (template-url (woo-get-option result 'url))
	 (scm (old-template-port template-url)))
    (and scm
	 (acons 'scm scm
		(plist->alist (cdr result))))))

;;;
(define (process-template url url-args)
  (or (new-template url)
      (old-template url url-args)
      (woo-throw "No template information provided for url " url)))
