(define-module (alterator http utils)
  :use-module (vhttpd)

  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-13)

  :use-module (alterator algo)
  :use-module (alterator str)
  
  :export (uri-prefix?
           decode-languages
           decode-cookie
           decode-content
	   decode-url
           decode-multipart

           encode-url-component))

;;; uri
(define (uri-prefix? prefix-uri main-uri)
  (and (string-prefix? prefix-uri main-uri)
       (let ((prefix-length (string-length prefix-uri))
             (main-length (string-length main-uri)))
         (or (= prefix-length 1)
             (= prefix-length main-length)
             (char=? (string-ref main-uri prefix-length) #\/)))))

;;; common splitters
(define (line-trim x)
  (string-trim-both x #\space))

(define (line-param x)
  (let ((x (map line-trim (string-cut-repeated x #\=))))
    (cons (car x) (cond-cadr x))))

(define (line-split x)
  (map line-param (string-cut-repeated x #\;)))

;;;;;;; cookies

(define (decode-cookie cookie)
  (if (string? cookie)
      (filter cdr (line-split cookie))
      '()))

;;;;;;; extract languages

;;extract q from parameters
;;FIXME: use line-split
(define (get-q params)
  (let ((result
         (find (lambda(param)
                 (and (= (length param) 2)
                      (string=? (car param) "q")))
               (map (lambda(item)
                      (string-cut item #\=))
                    params))))
    (if result
        (string->number (cadr result))
        1)))

(define (compare-q x y)
  (> (cdr x) (cdr y)))

;;FIXME: use line-split
(define (decode-languages langlist)
  (and (string? langlist)
       (map car
            (stable-sort
             (map (lambda (item)
                    (let ((attr (string-cut item #\;)))
                      (cons (car attr) (get-q (cdr attr)))))
                  (string-cut (string-delete langlist #\space) #\,))
             compare-q)) ))

;;; url decoding
(define (decode-url-arg x)
  (let ((x (string-cut x #\=)))
    (if (< (length x) 2)
        (error "wrong url component" x)
        (cons (string-delete (decode-url-component (car x)) #\cr)
              (string-delete (decode-url-component (cadr x)) #\cr)))))

(define (decode-url s)
  (if (string-null? s)
      '()
      (map decode-url-arg (string-cut s #\&))))

;;; multipart decoding

;;returns : start-index,last-index
(define (make-next-part s boundary len)
  (lambda (start)
    (let loop ((idx (string-contains s boundary start)))
      (and idx
           (let ((last-idx (+ idx len)))
             (cond
              ;;intermediate boundary
              ((string-contains s (string #\newline) last-idx)
               (cons
                (substring s start idx)
                (+ last-idx 1)))
              ;;final boundary
              ((string-contains s (string #\- #\- #\newline) last-idx)
               (cons (substing s start idx)
                     (+ last-idx 3)))
              (else (loop (string-contains s boundary last-idx)))))))))

(define (multipart-split s boundary)
  (and boundary
       (let* ((boundary-len (string-length boundary))
              (next-part (make-next-part s boundary boundary-len)))
         (let loop ((result '())
                    (start (+ boundary-len 1))) ;;skip first boundary with #\newline final character
           (let ((next (next-part start)))
             (if (pair? next)
                 (loop
                  (cons (car next) result)
                  (cdr next))
                 (reverse result)))))))

(define (multipart-boundary type)
  (string-append "--" (cond-assoc "boundary" (cdr type) "")))

(define (multipart-name s)
  (and-let* ((idx (string-contains-ci s "Content-Disposition:"))
             (last-idx (string-index s #\newline idx)))
            (string-trim-both 
             (cond-assoc "name"
                         (line-split (substring s idx last-idx))
                         "unknown")
             #\")))

;;TODO: recognize content-type of the fragment
(define (multipart-data s)
  (and-let* ((idx (string-contains s (string #\newline #\newline))))
            (string-drop-right (substring s (+ idx 2)) 1)))

(define (decode-multipart s type)
  (map (lambda(x)
         (cons (multipart-name x)
               (multipart-data x)))
       (multipart-split (string-delete s #\cr) (multipart-boundary type))))

;;; common decoding wrapper
(define (decode-content type s)
  (if (string-null? s)
      '()
      (let* ((type (line-split type))
             (type-name (caar type)))
        (cond
         ((string-ci=? type-name "application/x-www-form-urlencoded")
          (decode-url s))
         ((string-ci=? type-name "multipart/form-data")
          (decode-multipart s type))
         (else (error "Unsupported content type"))))))

;;; encoding
(define (char->hex c)
 (number->string
   (char->integer c) 16))

(define (encode-url-component str)
  (apply string-append
	 (map (lambda (c)
		(cond 
		  ((or (and (char>=? c #\a) (char<=? c #\z))
		       (and (char>=? c #\A) (char<=? c #\Z))
		       (and (char>=? c #\0) (char<=? c #\9)))
		   (string c))
		  ((char=? c #\space)
		   "+")
		  (else
		    (string-append "%" (char->hex c)))))
	      (string->list str))))


