(define-module (alterator workflow acc)
    :use-module (srfi srfi-1)
    :use-module (srfi srfi-13)
    :use-module (alterator algo)
    :use-module (alterator woo)

    :use-module (alterator http utils)
    :use-module (alterator http template)
    :use-module (alterator http html)

    :use-module (alterator configd woo)
    :use-module (alterator configd form)
    :use-module (alterator configd translate)
    :use-module (alterator configd html)
  :export (workflow))

(define (skip-scm scm url-args)
  (let ((head (cond-car scm)))
    (cond
      ((cond-assoc "async" url-args) scm)
      ((eq? head 'quote) (cdr scm))
      ((eq? head 'redirect) scm)
      (else #f))))

(define (scm-content scm name)
 (call-with-current-continuation
   (lambda(return)
    (template
      scm (tag: name (lambda (options content) (return content))))
    '())))

(define (help-tag url help)
  (replace-tag: "div"
		(@ 'id "help")
		(lambda (options content)
		  `(div ,@options
			,(@ 'style (format #f "visibility:~A;"
					   (if (string=? "/help" url) "hidden" "visible")))
			,(html: "a"
				(@ 'href (format #f "/help?topic=~A" (encode-url-component help)))
				content)))))

(define (acl-tag url desktopfile)
  (replace-tag: "div"
		(@ 'id "acl")
		(lambda (options content)
		  `(div ,@options
			,(@ 'style (format #f "visibility:~A;"
					   (if (member url '("/" "/ahttpd/acl")) "hidden" "visible")))
			,(html: "a"
				(@ 'href (format #f "/ahttpd/acl?desktopfile=~A" (encode-url-component desktopfile)))
				content)))))


(define (main-tag scm)
  (and (pair? scm)
    (tag: "div" (@ 'id "main") (scm-content scm "body"))))

(define (title? x)
  (and (pair? x) (eq? (car x) 'title)))

(define (head-tag scm title)
  (and (pair? scm)
    (replace-tag: "head"
		  (lambda(options content)
		    (list 'head
			  (remove title? content)
			  (remove title? (scm-content scm "head"))
			  (html: "title" title))))))

(define (title-tag title)
    (tag: "span" (@ 'id "hostinfo") title))

(define (apply-po-domain scm po-domain)
  (if (string? po-domain)
    (template scm
	      (replace-tag: "html" (lambda (options content)
				     `(html ,@options ,@content
					    ,(@ 'po-domain po-domain)))))
    scm))

;;; menu

(define (current-module? item url)
  (uri-prefix? (woo-get-option item 'uri) url))

(define (find-module url module-data)
  (find (lambda(x) (current-module? x url))
        module-data))

(define (filter-module category module-data)
  (filter (lambda(x) (string=? category (woo-get-option x 'category)))
        module-data))

(define (html:section group-name group-title items)
  (html: "div"
	 (@ 'class "menu-group")
	 (html: "div"
		(@ 'class "menu-group-name")
		(@ 'title group-title)
		group-name)
	 items))

(define (html:module item url)
  (html: "div"
	 (@ 'class (if (current-module? item url) "menu-item-selected" "menu-item-normal"))
	 (html: "a" (@ 'href (woo-get-option item 'uri))
                    (woo-get-option item 'name))))

(define (menu-tag url sections-data module-data)
  (tag: "div" (@ 'id "menu")
	(filter-map (lambda(section)
		      (let ((modules (filter-module (woo-get-option section 'category) module-data)))
			(and (pair? modules)
			     (html:section
			       (woo-get-option section 'name)
			       (woo-get-option section 'comment)
			       (map (lambda(item) (html:module item url))
			            modules)))))
		     sections-data)))

;;; main
(define (workflow-acc url url-args template-args)
  (let* ((scm (cond-assq 'scm template-args))
	 (scm2 (cond-assq 'scm2 template-args))
	 (po-domain (and (pair? scm2) (find-po-domain scm2)))
	 (language (cond-assoc "language" url-args '("en_US")))

	 ;;new menu
	 (sections-data (woo-list "/menu/avail_sections" 'language language 'ui "html"))
	 (module-data (woo-list "/menu/avail_modules" 'language language 'ui "html"))
	 (current-module (find-module url module-data))
	 (current-module-title (woo-get-option current-module 'name))
	 (current-module-help (woo-get-option current-module 'help "acc-html"))
	 (current-module-desktopfile (woo-get-option current-module 'desktopfile))
	 )
    (or (skip-scm scm2 url-args)
       (apply-po-domain
	 (template scm
		   (help-tag url current-module-help)
		   (acl-tag url current-module-desktopfile)
		   (head-tag scm2 current-module-title)
		   (title-tag current-module-title)
		   (menu-tag url sections-data module-data)
		   (main-tag scm2))
	 po-domain))))

(define (workflow url url-args template-args)
 (woo-catch
  (thunk (workflow-acc url url-args template-args))
  (html:exception url-args)))
